/*###########################################################*/
/** \file aes_crypt.h
Autor                 : Bernd Ottmann             01/2012
*/  
/*###########################################################*/
#pragma once
#ifndef DATAFOX_COMMUNICATION_AES_CRYPT_H
#define DATAFOX_COMMUNICATION_AES_CRYPT_H

#include <cstdint>

namespace Datafox
{
    namespace Communication
    {
        // Klasse zur Verschluesselung von Daten mittels AES.
        class Aes_crypt
        {
        public:
            Aes_crypt()
            {
                m_use_init_vectors_aes = 1;
            }
        private:
            static const uint8_t m_sbox[256];
            static const uint8_t m_inv_sbox[256];
            static const uint8_t m_ff_pow[256];
            static const uint8_t m_ff_log[256];
            static const uint8_t m_crc_tab[256];
            static const uint8_t m_aes_start_crc_value = 0x53;
            uint8_t m_default_key_aes[16]; ///< Schluessel für Verschluesselung
            uint8_t m_init_vector_encrypt_aes[16]; ///< Speicher fuer Initialisierungs Vektor
            uint8_t m_init_vector_decrypt_aes[16]; ///< Speicher fuer Initialisierungs Vektor
            uint8_t m_use_init_vectors_aes;  ///< benutze Initialisierungs Vektor beim Verschluesseln
            uint8_t m_round_keys_aes[176]; ///< Speicher fuer die einzelnen Round Keys

        public:
            // High Level Funktionen fuer die Verschluesselung
            void set_key( uint8_t *key_ptr, uint8_t size );
            uint16_t encrypt( uint8_t *data, uint16_t raw_size ); // Verschluesselung von Nutzdaten
            uint32_t encrypt_stream( uint8_t *data, uint32_t raw_size ); // Verschluesselung von Nutzdaten
            uint8_t decrypt( uint8_t *data, uint16_t *size );    // Entschluesselung von Nutzdaten
            uint8_t decrypt_stream( uint8_t *data, uint32_t *size );    // Entschluesselung von Nutzdaten

        private:
            // Checksumme fuer Verschluesselungsbereich berechnen
            uint8_t crc_check_data( uint8_t crc_previous, uint32_t size, uint8_t *data );
            // Blockweise
            void encrypt_block16( uint8_t *block ); // Verschluesselung von 16Byte Nutzdaten
            void decrypt_block16( uint8_t *block ); // Entschluesselung von 16Byte Nutzdaten
            // Helper, interne Berechnungsfunktionen
            void expand_key( uint8_t *input_key_ptr );
            void x_or( uint8_t *state, uint8_t *round_key, uint8_t size = 16 );
            void shift_rows( uint8_t *block );
            void inv_shift_rows( uint8_t *block );
            uint8_t ff_mul( uint8_t a, uint8_t b );
            void mix_column( uint8_t *block );
            void inv_mix_column( uint8_t *block );
        };
    } // namespace Communication
} // namespace Datafox

#endif
