/**
* @file  FileHardwareInfo.h
* @brief Schnittstelle fuer die Klasse FileHardwareInfo.
*/
//////////////////////////////////////////////////////////////////////
#ifndef FILEHARDWAREINFO_H
#define FILEHARDWAREINFO_H

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif

// Verwaltungsklasse fuer Datei
#include "file_intel_hex.h"
// 16bit/32bit CRC
#include "common/crc.h"

// Hilfsklassen für das Verarbeiten der Dateidaten.
#include "interface/hardware_info_items.h"

#include <memory>

class FileHardwareInfo : public FileIntelHex
{
public:
    FileHardwareInfo();
    FileHardwareInfo( const FileHardwareInfo &hwInfo );
    virtual ~FileHardwareInfo();

public:
    FileHardwareInfo &operator=( const FileHardwareInfo &hwInfo );

public:
    void reset();

public:
    bool load( const std::string &filePath );
    bool save( const std::string &filePath, bool appendToFile = false, bool appendEndLine = true );
    bool readSerialize();
    bool writeSerialize();

    bool isModified() 
    {
        return (_crc != checksum32bit());
    }

private:
    unsigned int _crc;

public:
    ItemDeviceHardwareInfo &rootItemDeviceHardwareInfo()
    {
        return _deviceHardwareInfo;
    }
private:
    // Verwaltungsstrukturen fuer HardwareInfo
    ItemDeviceHardwareInfo _deviceHardwareInfo;

public:
    listItemBoard collectModuleIds( const setModuleId &ids );
    // Shorties
    listItemBoard moduleRS232Communication( const setModuleRS232Communication &module = moduleRS232Communication_all );
    listItemBoard moduleRS485Communication( const setModuleRS485Communication &module = moduleRS485Communication_all );
    listItemBoard moduleUSBCommunication( const setModuleUSBCommunication &module = moduleUSBCommunication_all );
    listItemBoard moduleLANCommunication( const setModuleLANCommunication &module = moduleLANCommunication_all );
    listItemBoard moduleWLANCommunication( const setModuleWLANCommunication &module = moduleWLANCommunication_all );
    listItemBoard moduleMobileCommunication( const setModuleMobileCommunication &module = moduleMobileCommunication_all );
    listItemBoard moduleBluetoothCommunication( const setModuleBluetoothCommunication &module = moduleBluetoothCommunication_all );
    listItemBoard moduleMMCCommunication( const setModuleMMCCommunication &module = moduleMMCCommunication_all );
    listItemBoard moduleUSBHostCommunication( const setModuleUSBHostCommunication &module = moduleUSBHostCommunication_all );

    listItemBoard moduleAccessControl( const setModuleAccessControl &module = moduleAccessControl_all );
    listItemBoard moduleBiokey( const setModuleBiokey &module = moduleBiokey_all );
    listItemBoard moduleGPS( const setModuleGPS &module = moduleGPS_all );

    listItemBoard moduleDisplay( const setModuleDisplay &module = moduleDisplay_all );
    listItemBoard moduleTouch( const setModuleTouch &module = moduleTouch_all );

public:
    unsigned short (*signing)( unsigned char *data, std::vector<unsigned char>::size_type size );

public:
    void setAutoCorrectModulesDisabled( bool state );
};

#endif

